
@dlibrary pgraph;@
@procedures@



@*******************************************************
******************proc: find date***********************
output: y is the row of the date your looking for in the matrix of dates called dates.
indput: dates: is a matrix of dates with months in col1, year in column2
           month and year: specify the date of interest
*******************************************************@
	proc(1) = finddate(dates,month,year);
	local ind1, ind2, ind3, y, ind1a,ind1b,ind2a,ind2b;
	ind1a = dates[.,1] .> month - 0.1; ind1b = dates[.,1] .< month + 0.1;  
        ind1 = ind1a .* ind1b;@ones for correct month@
        ind2a = dates[.,2] .> year - .01; ind2b = dates[.,2] .< year + .01;    
	ind2 = ind2a .* ind2b;   @ones for correct year@
	ind3 = ind1.*ind2;                  @ones for correct month and year@
	y = ind3'seqa(1,1,rows(dates)); @pulls out row number of correct date@
	retp(y);
	endp;

@************************************************************************************
**proc: drim2
**This loads data from ../../data/exchange/dridrimonthly2.fmt and defines spot return, interest rate and forward premim
**variables from it
**format:
**{S, F, sr, fp, id, dates, name} = drim2(i,c1);
**input: 
**	i -- specifies currency: Australia, Canada, France, Germany, Japan, UK
**	c - choice of sample: c=1: dates choosen to match interest differential and spot return (F,fp=0),
**					  c1=2: dates choosen to match forward premium and spot return (id=0), 
**					  c1=3 dates choosen to mach spot return and forward premium 		
**      cc1 - compounding convention:
**           0 -no adjustment
**              i_month =1/1200*i_yr
**           1 -adjust using Bauer (01) footnote 9:
**              i_month = #days/360**1/100i_yr all but UK
**              i_month = #days/365**1/100i_yr for UK
**outputs:
**S -spot rate level (no logs)
**F - forward rate level (no logs)
**sr - spot return
**fp - forward premium
**id - interest differential
**dates - [month, year]
*************************************************************************************@
proc(7) = drim2(i,c1,_printout);
local S1, S2, S3,F2, F3, sr1, sr2, sr3, fp2, fp3, id1, id3, dates1, dates2, dates3,data, temp, id, S, F,name;
	load data = ../../data/exchange/dri/drimonthly2; 
@print out country names@
  "  ";  if i ==1; name = "AUD" ; " "; elseif i==2; name = "CAD"; " "; elseif i==3; name = "FFR";
  " "; elseif i==4; name = "GDM"; " "; elseif i==5; name = "JPY"; " "; elseif i==6; name = "BRP"; " ";   
endif;
/*" "; print name;  " ";*/
@compounding convention@

 @pull up series@
  id = 1/1200*(data[.,3]-data[.,3+i]);  @nominal interest differential: us - foreign@
  S  = data[.,3+6+i]; @Spot rate: dollar price of foreign currency@
  F =  data[.,3+12+i]; @forward rate@
 @get rid of missing values@
  temp = S~id~data[.,1:2];
  temp = packr(temp);
  S1 = temp[.,1];
  id1  = temp[.,2];
  dates1 = temp[.,3:4];  @dates1: col1: months  col2: year@
  temp = S~F~data[.,1:2];
  temp = packr(temp);
  S2 = temp[.,1];
  F2 = temp[.,2];
  dates2 = temp[.,3:4];  @dates2: col1: months  col2: year@
  temp = S~F~data[.,1:2]~id;
  temp = packr(temp);
  S3 = temp[.,1];
  F3 = temp[.,2];
  dates3 = temp[.,3:4];  @dates3: col1: months  col2: year@
  id3 = temp[.,5];
  @form returns variables@
  sr1 = ln(S1[2:rows(S1)])-ln(S1[1:rows(S1)-1]);  @spot return whose rows match interest differential@
  sr2 = ln(S2[2:rows(S2)])-ln(S2[1:rows(S2)-1]);  @spot return whose rows match forward premium@
  sr3 = ln(S3[2:rows(S3)])-ln(S3[1:rows(S3)-1]);  @spot return: where rows of fp and id forced to match@
  fp2 = ln(F2) - ln(S2);  @forward premium: using all data available@
  fp3 = ln(F3) - ln(S3);  @forward premium: whose rows match those of nominal interest differential@
  fp2  = fp2[1:rows(fp2)-1];
  fp3  = fp3[1:rows(fp3)-1];
  id1 = id1[1:rows(id1)-1];
  id3 = id3[1:rows(id3)-1];
if c1 == 1;
  retp(S1,0, sr1,0, id1, dates1, name);
	if _printout == 0; "dates1: from " dates1[1,.] " to " dates1[rows(dates1),.] ; endif;
  elseif c1 == 2;
  retp(S2, F2, sr2, fp2, 0, dates2, name);	
	if _printout == 0; "dates2: from " dates2[1,.] " to " dates2[rows(dates2),.] ; endif;
  elseif c1 == 3;
  retp(S3, F3, sr3, fp3, id3, dates3, name);	
	if _printout == 0; "dates3: from " dates3[1,.] " to " dates3[rows(dates3),.] ; endif;
endif;
endp;
@*******************************************@


@*****************************************************************
**proc: drim9
**This loads data from ../../data/exchange/dridrimonthly10.fmt 
** and defines spot return, interest rate and forward premim
**variables from it
**format:
**{S, F, sr, fp, id, dates, name} = drim9(i,c1,cc1);
**input: 
** i -- specifies currency: Australia, Canada, France, Germany, Japan, UK
**  c1 - choice of sample: c=1: dates choosen to match 
**       interest differential and spot return (F,fp=0),
**  c1=2: dates choosen to match forward premium and spot return (id=0), 
**  c1=3 dates choosen to mach spot return and forward premium  		
** cc1 - compounding convention:
**    0 -no adjustment
**    i_month =1/1200*i_yr
**    1 -adjust using Bauer (01) footnote 9:
**    i_month = #days/360**1/100i_yr all but UK
**     i_month = #days/365**1/100i_yr for UK
**outputs:
**S -spot rate level (no logs)
**F - forward rate level (no logs)
**sr - spot return  s_t+1 -s_t
**fp - forward premium f_t -s_t   
**id - interest differential id_t
**dates - [month, year]
** Note that s_t+1-s_t lines up with f_t-s_t and with id_t
********************************************************************@
proc(7) = drim9(i,c1,cc1,_printout);
local S1, S2, S3,F2, F3, sr1, sr2, sr3, fp2, fp3, 
   id1, id3, dates1, dates2, dates3,data, temp, id, S, F,name;
local leap,feb,day30, day31,ndim,term;
local month,day,year,month1,day1,year1,month2,day2,year2,month3,day3,year3;

        load data = ../../data/exchange/dri/drimonthly9;
@print out country names@
  "  ";  if i ==1; name = "AUD" ; " "; 
    elseif i==2; name = "CAD"; " "; 
    elseif i==3; name = "FFR";
     " "; elseif i==4; name = "GDM"; " "; 
    elseif i==5; name = "JPY"; " "; 
   elseif i==6; name = "BRP"; " ";   
endif;
/*" "; print name;  " ";*/



 @pull up series@
month = data[.,1];
day     = data[.,2];
year    = data[.,3];

S = data[.,10+i];  @Spot rate: dollar price of foreign currency@
F = data[.,16+i];  @forward rate@
@compounding convention@
if cc1 <=0; @no compounding@
  data[.,4:10]  = 1/1200*data[.,4:10];
elseif cc1 ==1;
  leap       =     year/4 .==round(year/4);
  feb         =     month.==2;
  day30    = (month.==4)+(month.==6)+(month.==9)+(month.==11);
  day31     = 1 -feb-day30;
  
ndim = 31.*day31+30.*day30+28.*feb+feb.*leap; @number of (calendar) days in month@
ndim = ndim|30;  @day n+1 is 4/2000 which has 30 days@
 term = ndim-day+(day[2:rows(day)]|30);  
@term (in days) of interestest contract (i.e. number of days from last
businessday in in this month to last business day in next month = days
remaining this month + no. bus. days in next month.  Assume 30
business days in month n+1.  This will get out below anyway.@
				
  data[.,4] =1/100*(term/360).*data[.,4];  @US rate adjustment@ 	
  if i==6;  @UK convention is 365 days@
    data[.,4+i] = 1/100*(term/365).*data[.,4+i];  @UK rate adjustment@ 	
  else;   @Other rates convention is 360@	
    data[.,4+i] = 1/100*(term/360).*data[.,4+i];  @Other rate adjustments@ 
  endif;

else;
  "ERROR: Set cc1 = 0 or 1 to use drim9";
endif;
id = (data[.,4]-data[.,4+i]); @nominal interest differential: us - foreign@


 @get rid of missing values@
  temp = S~id~month~day~year;
  temp = packr(temp);
  S1 = temp[.,1];
  id1  = temp[.,2];
   month1 = temp[.,3];
   day1 = temp[.,4];
   year1 = temp[.,5];
  dates1 =month1~year1;    @dates1: col1: months  col2: year@
  temp = S~F~month~day~year;
  temp = packr(temp);
  S2 = temp[.,1];
  F2 = temp[.,2];

   month2 = temp[.,3];
   day2 = temp[.,4];
   year2 = temp[.,5];
  dates2 = month2~year2;  @dates3: col1: months  col2: year@
  temp = S~F~id~month~day~year;
  temp = packr(temp);
  S3 = temp[.,1];
  F3 = temp[.,2];  
  id3 = temp[.,3];
  month3 = temp[.,4];
  day3 = temp[.,5];
   year3 = temp[.,6];
  dates3 = month3~year3;  @dates3: col1: months  col2: year@
  @form returns variables@ 
  sr1 = ln(S1[2:rows(S1)])-ln(S1[1:rows(S1)-1]);  
        @spot return whose rows match interest differential@
  sr2 = ln(S2[2:rows(S2)])-ln(S2[1:rows(S2)-1]);  
        @spot return whose rows match forward premium@
  sr3 = ln(S3[2:rows(S3)])-ln(S3[1:rows(S3)-1]);  
         @spot return: where rows of fp and id forced to match@
  fp2 = ln(F2) - ln(S2);  
          @forward premium: using all data available@
  fp3 = ln(F3) - ln(S3);  
        @forward premium: whose rows match those of nominal interest differential@
  fp2  = fp2[1:rows(fp2)-1];
  fp3  = fp3[1:rows(fp3)-1];
  id1 = id1[1:rows(id1)-1];
  id3 = id3[1:rows(id3)-1];
if c1 == 1;
  retp(S1,0, sr1,0, id1, dates1, name);
     if _printout == 0; "dates1: from " dates1[1,.] " to " dates1[rows(dates1),.] ; endif;
  elseif c1 == 2;
  retp(S2, F2, sr2, fp2, 0, dates2, name);	
     if _printout == 0; "dates2: from " dates2[1,.] " to " dates2[rows(dates2),.] ; endif;
  elseif c1 == 3;
  retp(S3, F3, sr3, fp3, id3, dates3, name);	
	if _printout == 0; "dates3: from " dates3[1,.] " to " dates3[rows(dates3),.] ; endif;
endif;
endp;
/***************************************************************/




/*************************************************
**proc: drim10

** This loads data from ../../data/exchange/dri/drimonthly10 
** and defines spot return, interest rate and forward premim
** variables from it
** format:
** {S, F, sr, fp, id, dates, name} = drim10(i,c1,cc1,computer,_printout);
**inputs:
** 
**i -- specifies currency: Australia, Canada, France, Germany, Japan, UK
**c1 - choice of sample: 
**     c1=1: dates choosen to match interest differential and spot return (F,fp=0),
**     c1=2: dates choosen to match forward premium and spot return (id=0), 
**     c1=3 dates choosen to mach spot return and forward premium  		
**cc1 - compounding convention:
**      0 -no adjustment
**         i_month =1/1200*i_yr
**      1 -adjust using Bauer (01) footnote 9:
**         i_month = #days/360**1/100i_yr all but UK
**         i_month = #days/365**1/100i_yr for UK
**outputs:
**
**S  -    spot rate level (no logs)
**F  -    forward rate level (no logs)
**sr -    spot return  s_t+1 -s_t
**fp -    forward premium f_t -s_t   
**id -    interest differential id_t
**dates - [month, year]
** Note that s_t+1-s_t lines up with f_t-s_t and with id_t
************************************************************/
proc(7) = drim10(i,c1,cc1,computer,_printout);
local S1, S2, S3,F2, F3, sr1, sr2, sr3, fp2, fp3, id1, id3, 
   dates1, dates2, dates3,data, temp, id, S, F,name;
local leap,feb,day30, day31,ndim,term;
local month,day,year,month1,day1,year1,month2,day2,year2,month3,day3,year3;

if computer $=="e177"; @old Dell@
        load data = d:\data\exchange\dri\drimonthly10;
elseif computer $=="e168_amaynard";  @Spadina St. AMD amaynard use@
        load data = ../../data/exchange/dri/drimonthly10;
elseif computer $=="e168_wei";  @Spadina St. AMD wei use@
        load data = /share2/data/exchange/dri/drimonthly10;
endif;

@print out country names@
  "  ";  if i ==1; name = "AUD" ; " "; elseif i==2; name = "CAD"; " "; 
        elseif i==3; name = "FFR";
  " "; elseif i==4; name = "GDM"; " "; elseif i==5; name = "JPY"; " "; 
   elseif i==6; name = "BRP"; " ";   

endif;
/*" "; print name;  " ";*/

 @pull up series@
month = data[.,1];
day     = data[.,2];
year    = data[.,3];

S = data[.,10+i];  @Spot rate: dollar price of foreign currency@
F = data[.,16+i];  @forward rate@
@compounding convention@
if cc1 <=0; @no compounding@
  data[.,4:10]  = 1/1200*data[.,4:10];
elseif cc1 ==1;
  leap       =     year/4 .==round(year/4);
  feb         =     month.==2;
  day30    = (month.==4)+(month.==6)+(month.==9)+(month.==11);
  day31     = 1 -feb-day30;
  
ndim = 31.*day31+30.*day30+28.*feb+feb.*leap; @number of (calendar) days in month@

 term = ndim-day+(day[2:rows(day)]|30);  @term (in days) of interestest 
        contract (i.e. number of days from last businessday in 
        in this month to last business day in next month = days 
        remaining this month + no. bus. days in next month.  Assume 30 
	business days in month n+1.  This will get out below anyway.
        day n+1 is 4/2000 which has 30 days: This is why the extra 30
        is tagged onto the end of the formula@
				
  data[.,4] =1/100*(term/360).*data[.,4];  @US rate adjustment@ 	
  if i==6;  @UK convention is 365 days@
    data[.,4+i] = 1/100*(term/365).*data[.,4+i];  @UK rate adjustment@ 	
  else;   @Other rates convention is 360@	
    data[.,4+i] = 1/100*(term/360).*data[.,4+i];  @Other rate adjustments@ 
  endif;

else;
  "ERROR: Set cc1 = 0 or 1 to use drim9";
endif;
id = (data[.,4]-data[.,4+i]); @nominal interest differential: us - foreign@


 @get rid of missing values@
  temp = S~id~month~day~year;
  temp = packr(temp);
  S1 = temp[.,1];
  id1  = temp[.,2];
   month1 = temp[.,3];
   day1 = temp[.,4];
   year1 = temp[.,5];
  dates1 =month1~year1;    @dates1: col1: months  col2: year@
  temp = S~F~month~day~year;
  temp = packr(temp);
  S2 = temp[.,1];
  F2 = temp[.,2];

   month2 = temp[.,3];
   day2 = temp[.,4];
   year2 = temp[.,5];
  dates2 = month2~year2;  @dates3: col1: months  col2: year@
  temp = S~F~id~month~day~year;
  temp = packr(temp);
  S3 = temp[.,1];
  F3 = temp[.,2];  
  id3 = temp[.,3];
  month3 = temp[.,4];
  day3 = temp[.,5];
   year3 = temp[.,6];
  dates3 = month3~year3;  @dates3: col1: months  col2: year@
  @form returns variables@ 
  sr1 = ln(S1[2:rows(S1)])-ln(S1[1:rows(S1)-1]);  @spot return whose rows match interest differential@
  sr2 = ln(S2[2:rows(S2)])-ln(S2[1:rows(S2)-1]);  @spot return whose rows match forward premium@
  sr3 = ln(S3[2:rows(S3)])-ln(S3[1:rows(S3)-1]);  @spot return: where rows of fp and id forced to match@
  fp2 = ln(F2) - ln(S2);  @forward premium: using all data available@
  fp3 = ln(F3) - ln(S3);  @forward premium: whose rows match those of nominal interest differential@
  fp2  = fp2[1:rows(fp2)-1];
  fp3  = fp3[1:rows(fp3)-1];
  id1 = id1[1:rows(id1)-1];
  id3 = id3[1:rows(id3)-1];
if c1 == 1;
  retp(S1,0, sr1,0, id1, dates1, name);
	if _printout == 0; "dates1: from " dates1[1,.] " to " dates1[rows(dates1),.] ; endif;
  elseif c1 == 2;
  retp(S2, F2, sr2, fp2, 0, dates2, name);	
	if _printout == 0; "dates2: from " dates2[1,.] " to " dates2[rows(dates2),.] ; endif;
  elseif c1 == 3;
  retp(S3, F3, sr3, fp3, id3, dates3, name);	
	if _printout == 0; "dates3: from " dates3[1,.] " to " dates3[rows(dates3),.] ; endif;
endif;
endp;
/*******************************************************************************************/




/*

/***************************************************************************
**                              lp_reg.g
****************************************************************************
**          Log Periodogram Regression: using first rrr2 values
**
**                        {d, b, Iww, se} = lp_reg(x,m)
**
**          d = estimated degree of fractional integration
**          b = 2 by 1 vector of slope & intercept coeffs from lp regression
**          Iww = Periodogram using 1st through rrr'th frequency
**          se = standard error of d
**          m - cut off parameter
****************************************************************************/

proc(4)= lp_reg(x,m);
local z,y,d,b,se, _output, vnam, m, stb, vc, stderr, sigma, cx, rsq, resid,
            dwstat, Iww, T;
t=rows(x);
Iww = periodo(x,m);


z = ln(2*pi*seqa(1,1,rows(Iww))/T);
y = ln(Iww);


/*
"----------";
print;
"Log Periodogram Regression Results";
"----------------------------------";
print;
*/
_output = 0;
b =  OLSQR(y,ones(rows(z),1)~z);
/*
print;
print;
"Estimated order of integration";
"------------------------------";
print;
"   d     standard error  ";
"----------------------- ";
print;
*/
d = -b[2]/2;
se = pi/sqrt(24)/sqrt(m);
/*print d~se;
print;
print;
*/

retp(d,b,Iww,se);
endp;
/************************************************************************/
*/

/***************************************************************************
**                             Periodo.g
**                             ---------
****************************************************************************
**                        Estimates the Periodogram
**                        -------------------------
**                        Iww = periodo(x, rrr)
**
**            Iww = periodogram  of x at frequencies 1/2pi to rrr/2pi
***************************************************************************/

proc periodo(x,rrr);
local Iww,xbar,gam0,gam,j,t,ttt,w;
t=rows(x);
/*Estimation of autocorrelations used in periodogram*/
xbar = 1/T*sumc(x);
gam0 = 1/T*(x-xbar)'(x-xbar);
gam = zeros(T-1,1);
j = 1;
do until j>t-1;
        gam[j] =1/T*(x[1+j:T]-xbar)'(x[1:T-j]-xbar);
        j = j+1;
endo;

/*Estimation of Periodogram at first rrr values*/
Iww = zeros(minc(rrr|floor(rows(x)/2)),1);
    /*periodogram at freq's 2pi/T to r*2pi/T */
ttt = 1;
do until ttt>minc(rrr|floor(rows(x))/2);
        w = 2*pi*ttt/T;
        Iww[ttt] = 1/(2*pi)*(gam0+2*gam'cos(w*seqa(1,1,T-1)));
        ttt = ttt+1;
endo;

retp(Iww);
endp;


/***************************************************************************
**                             longrv.g
**                             ---------
****************************************************************************
**{fxx,omeg,sig,lam,jam, lband2} = (x,y,smooth,lband,_kernel);
**
** Calculates Long run variances.  Defult is a QS kernel with 
automatic band width selection
**along the line of Andrews,91 (?? double check). No prefiltering 
employed (?? double check) series ,
_kernel = 4; 	Choice of kernel'
			" 	1 =   Quadratic Spectral (Andrews 91)  ";
			"       2 =   Parzen (also Gallant 87)";   
			"       3 =   Tukey-Hanning ";   
			"       4 =   Bartlett (also Newey-West 87)";   
 
** wx =0 long run var (use kernal =1)
** smooth: 0:  use automatic bandwidth
**   otherwize it gives rate at which the bandwidth expands 
**  (i.e. it replaces the optimal rate in the automatic
**   bandwidth selection procedures of Andrews.)  
**  This allows for the possibitlity of under smoothing.
**lband:  bandwidth parameter
**   0:  is automatic bandwidth using the value given by smooth
**   -1: sets the bandwidth equal to n^smooth 
**   (i.e. it obeys bandwidth rate but doesn't use optimal value 
**   any vale > 1:  will set the bandwidth to this precise value
**let eps_t = x_t
**            y_t
**sig = E(eps_0 eps_0') =  E(x_o^2) E(x_0 y_0)
**                         E(y_0 x_0) E(y_0^2)
**lam = sum_{h=1}^infty E(eps_0 eps_h') = lam_xx lam_xy
**                                        lam_yx lam_yy
**lam_xy NOT EQUAL lam_yx
**
**lam_xy = sum_{h=1}^infty E(x_0 y_h)     past x future y
**lam_yx = sum_{h=1}^infty E(x_h y_0)     past y future x
**
**jam = lam' = lam_xx lam_yx
**            lam_xy lam_yy
**
**Omega = sig+lam+lam' = Omega_xx Omega_xy  Long run variance
**                       Omega_xy Omega_yy
**NOTE: THESE DEFN'S ONLY HOLD TRUE AT FREQUENCY ZERO: i.e. wx =0
**
**fxx = estimate of spectrum at frequency wx
**
** lband2:  the bandwidth that was actually employed
************************************************************************/
   
proc(6) = longrv(x,y,smooth,lband,_kernel);
local eps,wx,e,nn,p,eb,ef,ae,ee,se,ad,a2,jb,j,xx,nw;
local jband,kern,ei,ej,sig,lam,jam,fxx,omeg,bandrt,lband2;
@Long--run variance/covariance 
estimation@
eps = x~y;
@The Long Run Calculations@
wx = 0;    @ estimation at frequency zero@
  e  = eps;
  xx  = wx;
  nn = rows(eps);
  p  = cols(eps);
  nw = cols(wx);
  eb = e[1:nn-1,.];
  ef = e[2:nn,.];
  ae = sumc(eb.*ef)./sumc(eb.^2);
  ee = ef - eb.*ae';
if lband ==0; @only needed for automatic bandwidth @
    se = meanc(ee.^2);
    ad = sumc((se./((1-ae).^2)).^2);
    a2 = 4*sumc((ae.*se./((1-ae).^4)).^2)/ad;
endif;
if smooth == 0; @automatic bandwidth@
  bandrt = .2; 
 @band rate: ONLY FOR Parzen, Tukey-Hanning, and QS kernels (not Bartlet)@
@NEW@  if lband == 4; @Bartlett@
@NEW@      bandrt = 1/3;
@NEW@  endif;
else;
 bandrt = smooth;  @allows for undersmoothing@
endif;
if lband>0;  @use pre-assigned bandwidth@
elseif lband == -1; @set lband = n^bandrt @
  lband =  nn.^bandrt;
elseif lband == 0; @use automatic bandwidth@
  if _kernel == 1;                               @  Quadratic Spectral @
    lband = 1.3221*((a2*nn).^bandrt);
  elseif _kernel == 2;                           @Parzen@
    lband = 2.6614*((a2*nn).^bandrt);            @automatic bandwidth@
@NEW@  elseif _kernel ==3;                       @Tukey-Hanning@
@NEW@  lband = 1.7462*((a2*nn).^bandrt);         @automatic bandwidth@ 
@NEW@  elseif _kernel ==4; 			 @Bartlett@
@NEW@   lband = 1.1447*((a2*nn).^bandrt);        @automatic bandwidth@
  endif;
endif;
lband2 = lband;

                                               @ Now estimate Covariances @
  jb = seqa(1,1,nn-1)/lband;
  if _kernel == 1;                               @ Quadratic Spectral 
Kernel @
    jband = jb*1.2*pi;
    kern = ((sin(jband)./jband - cos(jband))./(jband.^2))*3;
  elseif _kernel == 2;                            @  Parzen kernel @
    kern = (1 - (jb.^2)*6 + (jb.^3)*6).*(jb .<= .5);
    kern = kern + ((1-jb).^3).*(jb .<=1).*(jb .> .5)*2;
@NEW@  elseif _kernel ==3;                        @Tukey-Hanning@
@NEW@  kern = 0.5*(1+cos(pi*jb)).*(abs(jb) .<=1);
@NEW@  elseif _kernel ==4;                        @Bartlett@
@NEW@  kern = (1 -abs(jb)).*(abs(jb).<=1);
  endif;
  ei  = complex(cos(xx),sin(xx));
  ej  = complex(cos(-xx),sin(-xx));
  sig = (e'e/nn)*ones(1,nw);
  lam = zeros(1,nw);
  jam = zeros(1,nw);
  j   = 1;
  do while j <=nn-1;
     lam = lam + (e[1:nn-j,.]'e[1+j:nn,.])*kern[j].*ei/nn;
     jam = jam + (e[1:nn-j,.]'e[1+j:nn,.])'*kern[j].*ej/nn;
     j = j + 1;
  endo;
  fxx = real((sig + lam + jam)/(2*pi));  /*lam and jam are one sided 
long run covariances. may add sig. don't divide by the 2*pi*/
  sig = real(sig);
  lam = real(lam);
  jam = real(jam);
  omeg = sig+lam+jam;
retp(fxx,omeg,sig,lam,jam,lband2);
endp;
/*****************************************************/


/****************************************************
**			Combin.g
**      		---------
*****************************************************
**  Computes combinations: C(n,r) = n!/((n-r)!(r!))
** ------------------------------------------------
**
** C = combin(n,r);
**
** note: when n is small direct calculation is possible using the ! function
**  This program is needed for n> about 170
*******************************************************************/
proc(1) = combin(n,r);
local c,i,j,r1;
c =1;
i = n;  @count fron n!/(n-r)! in numerator@
j =1;  @count for r! in denomenator@
r1 =r;
if r1 >n/2;
	r1 = n-r1;  @always choose r to be smaller than 1-r@
endif;
do until i<n-r1+.5;  
	c = i*c;
	if j <r1+0.5;
		c = c/j;
	endif;
j = j+1;
i = i-1;
endo;
retp(c);
endp;
/******************************************************************/







/************************************************************************
**			cdfbi.g
**			--------------
************************************************************************
**  Computes Pr(X<x) for binomial(n,p) distribution
** --------------------------------------------------
**
** pr = cdfbi(x,n,p)
**
** pr = P(X<=x), n is no. of Bernoulli trials, p is prob of success
**
** note:  This calls combin.g
**************************************************************************/
proc(1) = cdfbi(x,n,p);
local pr, k, c1;
pr =0;
k =0;
do until k > x;  /*originally k>x*/
	c1 = combin(n,k);
	pr = pr+c1*p^k*(1-p)^(n-k);	
	k =k+1;
endo;
retp(pr);
endp;
/******************************************************************************/


/******************************************************************************
**		                      cdfbirev.g
**          		             --------------
*******************************************************************************
**	Computes smallest k for which Pr(X<k)>a for binomial(n,p) distribution
**	----------------------------------------------------------------------
**
**	k = cdfbirev(a,n,p)
**
**	prob(X<k)>a and k is smallest integer to satisfy this condition, 
**       n is no. of Bernoulli trials, p is prob of success
**
**	note:  This calls cfbi, which calls combin.g
********************************************************************************/
proc(1) = cdfbirev(a,n,p);
local pr, k, c1;

pr =1;
k =n;
do until pr < a;
	pr = cdfbi(k,n,p);
	k =k-1;
endo;
k = k+2; @adjusts for additional k = k-1, and the fact we overshot even before this@

retp(k);
endp;
/******************************************************************************/



/***************************************************************************
**		          	   signstat.g
**             	        	   ---------
****************************************************************************
**               Computes the Campbell-Dufour Sign Statistic
**         ----------------------------------------------------------
**  Under the null hypothesis that y_t  indep of y_t-1,y_t-2...., x_t-1,x_t-2,....
**  and the additional assumption that y_t is median zero, Sg is has a binomial(n,0.5) 
**  distribution. y = y_1,...y_n glag1 = g_0,...,g_n-1, g_t = g(x_t,x_t-1,...).  It may make sense 
**  demean or demedian y before running.  
**  {Sg,n,se,pr} = signstat(y,glag1,b)
**		
**  Sg is the sign statistic, which is binomial(n,0.5) under the null
**  n = rows(y)
**  se = standard deviation(Sg)
**  pr  =prob(X<Sg)  when X is binomial(n,0.5) 
**  (ie this provides a one sided test for a negative relationship between y and x)	
**        
**	
**   note:  100*Sg/n, 100*se/n - gives Sg (and se) is percent terms 
    (i.e. percent of positive y_t, g_t-1 pairs)
**	
**  note:  This calls cdfbi.g, which calls combin.g                      
**        
**   Reference: Cambell and Dufour, RESTAT 95 pp1-16,  Cambell & Dufour. IER, 97: 37(1)
***************************************************************************/
proc(4) = signstat(y,glag1,b);
local Sg,n,pr,v,se;
v = (y-b).*glag1 .> zeros(rows(y),1); 
   @indicator vector: 1 if (y_t-b) g_t-1 >0,  0 otherwise@
Sg = sumc(v); @number of y_t g_t-1 > 0 @
n = rows(y);

se = sqrt(n*.5*(1-.5)); @standard error of Sg under H_0@
/*
if Sg <= n*.5;
pr = cdfbi(Sg,n,0.5);
elseif Sg > n*.5;
pr = 1-cdfbi(Sg,n,0.5);
endif;
*/
/*pr = cdfbi(Sg,n,0.5); original from Alex*/
/****** use standard normal to approximate binomial******/
if Sg <= n*.5;
pr = 0.5-cdfn2(0,abs((Sg-n*.5)/se)); 
/*pr = cdfn((Sg-n*.5)/se);*/
elseif Sg > n*.5;
pr=0.5-cdfn2(0,(Sg-n*.5)/se);
/*pr = (1-cdfn((Sg-n*.5)/se));*/
endif;
/*******************************************************/

retp(Sg,n,se,pr);
endp;
/************************************************************************/


/***********************************************************************
                           signrankstat.g
*************************************************************************
** Computes the Campbell-Dufour Sign Rank Statistic tests for estimated median                   
** ---------------------------------------------------------------------------
***************************************************************************/

proc(1) = signrank(y, glag1, b);
local Srg,n,pr,v,u,u1,uprod;

v = (y-b).*glag1 .> zeros(rows(y),1); 
u = v~abs(y-b);

u1 = sortc(u,2);
n = seqa(1,1,rows(y));
u1 = u1~n;
uprod = prodc(u1[.,1 3]');
Srg = sumc(uprod);

retp(Srg);
endp;

/************************************************************************
**	            	        signstat2.g
**          		        ---------
****************************************************************************
**  Computes the Campbell-Dufour Sign Statistic tests for unknown median
**  -----------------------------------------------------------------------
**  Under the null hypothesis that y_t  indep of y_t-1,y_t-2...., x_t-1,x_t-2,....
**  and the additional assumption that y_t - b is median zero, Sg has a binomial(n,0.5) 
**  distribution. y = y_1,...y_n glag1 = g_0,...,g_n-1, g_t = g(x_t,x_t-1,...).  
**
**  {Sg,QL,QU,QBL,QBU,Sg_bar2,Sg_bar3,Sg_bar2B,Sg_bar3B) 
**      = signstat2(y,glag1,alpha,alpha1,_ngrid,name,graphit);
**
**  
**   Global Inputs:
**   _graphit:  0-no graph, 1, graph two sided test, 
**      2 graph left sided test, 3 graph right sided tests, 
**      4 graph both one sided tests together.
**   _ngrid:  number of grid points in the confidence interval J() over which Sg(b) is evaluated
**
**
**  y_t:  Series that is orthogonal to the past under the null, with unknown median b
**  glag1;  series which captures past information content
**  alpha:  size of the orthogonality test
**  alpha 1:  used to determine J(alpha1), an (1-alpha1) confidence 
**         region for the unknown median of y_t
**		
**   Sg is column of sign statistics evaluated over a grid of b in J(alpha1) 
**      the confidance interval for the unknown median of y_t. Under the null this 
**      statistic is binomial(n,0.5) when evaluated at the true median bo
**  n = rows(y)
**  QL:   min(sg(b)) for b in J(alpha1).  
**	  If QL > Sg_bar2 than we have a right sided rejection. 
**	  If QL >n-Sg_bar3 we fail to have a left sided rejection.
**  QU:   max(sg(b)) for b in J(alpha1). 
**	  If QU< Sg_bar3 than we fail to have a right sided rejection.
**	  If QU<n -Sg_bar2 than we have a left sided rejection.
**  QBL:   min over b of the larger of the two statitistics, Sg(b) or n-Sg(b). 
**	  If QBL > Sg_bar2B we have a two sided rejection
**  QBU:   max over b of the larger of the two statistics, Sg(b), n-Sg(b)
**	  IF QBU <Sg_bar3B we fail to provide a two sided rejection.
**
**   Sg_bar2:  the outer critical value for right sided tests.  
**        (for left sided tests use: n-sg_bar2)
**   Sg_bar3:  the inner critical value for right sided tests.  
**        (for left sided tests use: n-Sg_bar2)
**
**   Two sided tests using QL and QU:
**	  If QL>Sg_bar2B or if QU<n-Sg_bar2B we have a rejection
**	  If QU<Sg_bar3B AND QL > Sg_bar3B  fail to reject.
***	
**   note:  100*Sg/n, 100*se/n - gives Sg (and se) is percent terms 
**        (i.e. percent of positive y_t, g_t-1 pairs)
**	
**   note:  This calls med.g, signstat.g, and cdfbi.g, which calls combin.g                      
**        
**   Reference: Cambell and Dufour, RESTAT 95 pp1-16,  Cambell & Dufour. IER, 97: 37(1)
***************************************************************************/
proc(9) = signstat2(y,glag1,alpha,alpha1,_ngrid,name,_graphit,_printout);
local Sg,n,pr,v,se,ml,mu, Sg_bar2,Sg_bar3,b_grid,alpha2,alpha3,QL,QBL,QBU,QU,Sg_bar2B,Sg_bar3B;
local _paxht,_ptitlht,_pnumht,_pcolor,_pmcolor,_pdate,_plegstr,_pltype,_plwidth;

n = rows(y);
alpha2 = alpha -alpha1;
alpha3 = alpha+alpha1;
	@form CI for median of y_t@
{ml,mu} = med(y,alpha1);  @ml, mu are lower and upper bounds on J(alpha1), the CI for med(y_t)@
	@cut CI into a fine grid & calculate Sg at each point@
b_grid = seqa(ml, (mu-ml)/_ngrid, _ngrid);
v = (y*ones(1,rows(b_grid))-ones(rows(y),1)*b_grid').*(glag1*ones(1,rows(b_grid))) 
     .> zeros(rows(y),rows(b_grid)); @indicator vector: 1 if (y_t-b) g_t-1 >0,  0 otherwise@
Sg = sumc(v); @number of y_t g_t-1 > 0, each column of v corresponds to a different value of b@
	@define the supremem and infimum stats@
QL = minc(Sg);  @largest Sg stat: for one sided test@
QBL = minc(maxc(Sg'|(n-Sg'))); @largest Sg stat: for two sided test@
QBU = maxc(maxc(Sg'|(n-Sg'))); @smallest Sg stat: for two sided test@
QU = maxc(Sg);  @smallest Sg stat: for one sided test@
	@Define the critical values@

Sg_bar2 = cdfbirev(1-alpha2,n,0.5);
Sg_bar2B =  cdfbirev(1-alpha2/2,n,0.5); @balanced upper critical value: Sg^bar(alpha_2/2)@
Sg_bar3 = cdfbirev(1-alpha3,n,0.5);
Sg_bar3B =  cdfbirev(1-alpha3/2,n,0.5); @balanced lower critical value: Sg^bar(alpha_3/2)@



	@printout@
	format 4,4;
	"   ";
	if _printout ==0;
	  "Campbell & Dufour Sign stats with unknown median ";
	  "--------------------------------------------------";
 	  "  ";
	endif;
/*
	"right-hand one sided test results";
	"----------------------------------------------------";
	"QL, QU, Sg_bar2,Sg_bar3";
	print QL~QU~Sg_bar2~Sg_bar3;
	" ";
	"left-hand one sided test results";
	"----------------------------------------------------";
	"QL, QU,n-Sg_bar2,n-Sg_bar3";
	print QL~QU~(n-Sg_bar2)~(n-Sg_bar3);
	" ";
	"two-sided test results";
	"----------------------------------------------------";
	"QBL, QBU,n-Sg_bar2,n-Sg_bar3";
	print QBL~QBU~Sg_bar2B~Sg_bar3B;
*/
	" ";
	if _printout ==0;	
	   " Joint print out ";
	   "----------------------------------------------------";  
	  " sample size " n; 
	  " number of grid points " _ngrid ; 
	  " size of the test (alpha) "  alpha;
	  "size of rejection region corresponding to confidence interval for median of y (alpha1)  " alpha1; 
	   " tests stats:   right sided critical vales: left sided critical values ";
	endif;
	" & QL & QU & Sg_bar2 & Sg_bar3 &n-Sg_bar2 & n-Sg_bar3 ** ";
	" " name " & " QL " & " QU " & " Sg_bar2 " & " Sg_bar3 " & " n-Sg_bar2 " & " n-Sg_bar3 " ** ";
	" ";
	@size of rejection region corresponding to confidence interval for median of y@

	@Graphs@
	graphset;
/*@linux@_pqg_ps_orientation = "portrait";*/
/*@linux@_pqg_eps_orientation = "portrait";*/
/*@linux@_pqg_ps_rotate = 45;*/
	_paxht = .33;@axis height@
	_ptitlht = .35; @title height: use .38 for all six@
	_pnumht = .29; @tic marker height: use .32 for all six@
      _pcolor = 12; _pmcolor =12;@best color for importing into SWP@ _pdate = 0; 
           @remove gauss date stamp@
/*	_plegctl = {2,6,5.55,4.5}; 
              @set size and location of legend string:{coordinates, font size, x coord, y coord @  */
/*	_plegctl = {2,6,5.55,3.95}; 
             @set size and location of legend string:{coordinates, font size, x coord, y coord @  */
if _graphit == 3; @graph right-sided test@
	_plegstr = "S]g[(b) \000 S[u] \000 S[l])";	
	title(name);	
xlabel("b"); ylabel("S]g[(b)");
	xy(b_grid,Sg~(Sg_bar2*ones(rows(Sg),1))~(Sg_bar3*ones(rows(Sg),1)));
elseif _graphit ==2;  @graph left sided test@
	_plegstr = "S]g[(b) \000 n-S[u]]g[ \000 n-S[l]]g[)";	  
	title(name);
	xlabel("b"); ylabel("S]g[(b)");
	xy(b_grid,Sg~((n-Sg_bar2)*ones(rows(Sg),1))~((n-Sg_bar3)*ones(rows(Sg),1)));
elseif _graphit ==1; @Graph two sided test@
	_plegstr = "S]g[(b) \000 S[ub]  \000 S[lb]  \000 n- S[ub]) \000 n-S[lb]]g[ ";	 
	title(name);
	xlabel("b"); ylabel("S]g[(b)");
	xy(b_grid,Sg~(Sg_bar2B*ones(rows(Sg),1))~(Sg_bar3B*ones(rows(Sg),1))
            ~((n-Sg_bar2B)*ones(rows(Sg),1))~((n-Sg_bar3B)*ones(rows(Sg),1)));
elseif _graphit == 4; @Graph both one sided tests together 
          (looks liked two-sided test graph but critical values are different@
/*	_plegstr = "S]g[(b) \000 S[u] \000 S[l]  \000 n-S[u] \000 n-S[l]"; */
	_pltype = {6,1,3,1,3}; @line type@
	_plwidth  = {2,3,3,3,3};     @Control line thickness for main curves@
	title(name );
	xlabel("b"); ylabel("S]g[(b)");
	scale(b_grid ,((minc(Sg)-.5)*ones(rows(Sg),1))~((maxc(Sg)+.5)*ones(rows(Sg),1))
         ~((Sg_bar2+.5)
          *ones(rows(Sg),1))~(Sg_bar3*ones(rows(Sg),1))~((n-Sg_bar2-.5)
          *ones(rows(Sg),1))~((n-Sg_bar3)*ones(rows(Sg),1)) );
	xy(b_grid,Sg~(Sg_bar2*ones(rows(Sg),1))~(Sg_bar3*ones(rows(Sg),1))
           ~((n-Sg_bar2)*ones(rows(Sg),1))~((n-Sg_bar3)*ones(rows(Sg),1)));
endif;
graphset;
/*@linux@_pqg_ps_orientation = "portrait";*/
/*@linux@_pqg_eps_orientation = "portrait";*/
/*@linux@_pqg_ps_rotate = 45;*/
_pcolor = 12; _pmcolor =12;
retp(Sg,QL,QU,QBL,QBU,Sg_bar2,Sg_bar3,Sg_bar2B,Sg_bar3B);
endp;


/****************************************************************
**		         	  med.g
**             	     	        ---------
*******************************************************************
**  Provides an exact finite sample confidance interval for the sample
**  median using order statistics
**  -----------------------------------------------------------------
**
**       {ml,mu} = med(y,alpha)
**		
**        y is Sg is the sign statistic, which is binomial(n,0.5) under the null
**	   alpha: 1 - alpha is confidence level
**        ml, mu:  lower and upper bounds of alpha level confidence interval for median
**
**        note:  This calls cdfbi.g, which calls combin.g                      
**        
**	   Reference: Cambell & Dufour. IER, 97: 37(1),Hettmansperger, T.P. 
**        "Statistical inference based on ranks," New York: Wiley & Sons, 1984)
***************************************************************************/
proc(2) = med(y,alpha);
local n,k,pr,yy, ml,mu;
n = rows(y);
  @obtain lower bound confidence interval for binomial distribution@
pr = 0;
k =1;
do until pr >alpha/2;
	pr = cdfbi(k,n,0.5);
	k = k+1;
endo;
k = k-2;    @ minus 1 to reverse last k = k+1, minus 1 to get largest k s.t. pr<alpha/2@
   @order the y_t by rank@
yy = sortc(y,1);  @sorted by rank@
ml = yy[k+1];
mu = yy[n-k];
retp(ml,mu);
endp;
/************************************************************************/

/*********************************************************************
**	              	          med2.g
**          		        ---------
****************************************************************************
**  De-medians a series using only past values: ie computes y_t - med(y_1,...,y_t-1)
**  ------------------------------------------------------------------------
**
**       ydm = med2(y)
**		
**        y is original series
**	   ydm = y_t - med(y_1,...,y_t-1)
***************************************************************************/
proc(1) = med2(y);
local i, ydm;
ydm = zeros(rows(y),1);
ydm[1] = y[1];
i =2;
do until i>rows(y);
  /*ydm[i] = y[i] - median(y[1:i-1]);*/
  ydm[i] = y[i]-median(y[1:i]); /*adjusted based on CD1997*/
i = i+1;
endo;
retp(ydm);
endp;
/************************************************************************/








/************************************************************************/
/**                             Autocorrelogram **/
/****************************************************************************
**                     autoc (Autocorrelogram)
**                     ********************************
**                    It gives autocorrelogram and cross autocorrelogram
**                      for multivariate time series
**                    Format: {rho,gam} = autoc(x)
**                    Inputs: x:  T by k matrix (k time length T time series)
**                    Outputs:
**                             gam: T by k^2 matrix of autocovariances
**                                      gam[h+1,.] = vec(cov(X_t+h,X_t))'
**                             rho: T by k^2 matrix of autocorrelations
**                                      rho[h+1,.] = vec(cor(X_t+h,X_t))'
**
****************************************************************************/
proc(2) =  autoc(x);
local T, gam,rho,rho0,j,k, xbar, denom, joke,RR,QQ;
/*Estimation of autocorrelations used in periodogram*/
T= rows(x);
xbar = 1/T*sumc(x)';
k = cols(x);
gam = zeros(T,k^2); @autocovariances@
rho = zeros(T,k^2); @autocorrelations@
RR = 1/T*(x-xbar)'(x-xbar); @covariance matrix@

denom = sqrt(diag(RR))*sqrt(diag(RR)'); @Denominator of correlation coeff@
QQ = RR./denom;   @correlation matrix@
gam[1,.] =(vec(RR))';
rho[1,.] =(vec(QQ))';
j = 1;
do until j>T-1;
        RR = 1/T*(x[1+j:T,.]-xbar)'(x[1:T-j,.]-xbar);
        QQ = RR./denom;
        gam[j+1,.] = (vec(RR))';
        rho[j+1,.] = (vec(QQ))';
        j = j+1;
endo;
/*
@graphing correlograms@
library pgraph;
graphset;
/*@linux@_pqg_ps_orientation = "portrait";*/
/*@linux@_pqg_eps_orientation = "portrait";*/
/*@linux@_pqg_ps_rotate = 45;*/
_pcolor =12; _pmcolor =12; _pdate = 0;
j =1;
do until j>k^2;
         xy(seqa(0,1,rows(rho)),rho[.,j]);
j = j+1;
endo;
*/
retp(rho,gam);
endp;
/************************************************************************/


/************************************************************************
**                       y  =    round2(x,r)
**                       x  =    real number
**			 r  =    number of places after decimal point to keep
**                       y  =    rounded version of x out to r digits after decimal point
**************************************************************************/
proc(0)= round2(x,r);
local y;
format /rd 8,r;
y= 10^(-r)*trunc(round((10^r)*x));
print y;
format /rd 8,4; @return to previous format@
endp;

/*****************************************************************************************
                        ii = cswitch(i)
switches currency order for cross-currency regression
 i = integer between 1 and 6
ii = new integer between 1 and 6
****************************************************************************************/
proc cswitch(i);
local ii;
if i ==1;
  ii =4; @DM@
elseif i ==2;
  ii = 5; @JP@
elseif i==3;
  ii = 6; @GBP@
elseif i == 4;
  ii = 3; @FRA@
elseif i==5;
  ii =2; @CAD@
elseif i==6;
  ii = 1; @AUD@
endif;
retp(ii);
endp;
/*******************************************************************************************/
